package info.kwarc.mmt.frameit.rules

import info.kwarc.mmt.api.checking._
import info.kwarc.mmt.api.{DPath, GlobalName, Path}
import info.kwarc.mmt.api.frontend.Controller
import info.kwarc.mmt.api.objects._
import Conversions._
import info.kwarc.mmt.api.uom.{RealizedType, Simplifiability, Simplify, StandardInt, StandardPositive}
import info.kwarc.mmt.lf.ApplySpine
import info.kwarc.mmt.LFX.datatypes.{Listtype, Ls}
import info.kwarc.mmt.api.utils.URI

object GetListAtInd {
  val ns: DPath = DPath(URI.http colon "mathhub.info") / "FrameIT" / "frameworld"
  val path: GlobalName = ns ? "GetListAtInd" ? "getListAtInd"
  val rawhead: OMID = OMS(path)
  def apply(tp1: Term, ls: Term, inde: Term): Term =
    ApplySpine(this.rawhead, tp1, ls, inde)
  def unapply(tm : Term) : Option[(Term, Term, Term)] = tm match {
    case ApplySpine(this.rawhead, List(tp1, ls, inde)) => Some((tp1, ls, inde))
    case _ => None
  }
}

object GetListAtIndComp extends ComputationRule(GetListAtInd.path) {
  override def applicable(t: Term): Boolean = t match {
    case GetListAtInd(_, _, _) => true
    case _ => false
  }
  override def apply(check: CheckingCallback)(tm: Term, covered: Boolean)(implicit stack: Stack, history: History): Simplifiability = tm match {
      case GetListAtInd(_, Ls(lsi), inde) =>
        val p = Path.parseS("http://mathhub.info/MitM/Foundation?NatLiterals?pos_lit")
        val indexSimpl = check.simplify(inde)
        val indexBase = indexSimpl match {
          case OMLIT(i: BigInt, RealizedType(OMS(`p`), StandardPositive)) => Some(i)
          case _ => None
        }
        indexBase match {
          case Some(idx) =>
            val idxInt = idx.toInt
            if ((idxInt >= lsi.length) || (idxInt < 0)) {
              return Simplifiability.NoRecurse
            }
            Simplify(lsi(idx.toInt))
          case _ => Simplifiability.NoRecurse
        }
      case _ => Simplifiability.NoRecurse
  }
}